#!/usr/bin/env python

"""
update MDN documentation

see:
http://developer.mindtouch.com/en/ref/MindTouch_API/POST%3Apages%2F%2F%7Bpageid%7D%2F%2Fcontents

The manifest format is in the form:

mozrunner/README.txt  https://developer.mozilla.org/en/Mozrunner
jsbridge/README.txt   https://developer.mozilla.org/en/JSbridge
mozmill/README.txt    https://developer.mozilla.org/en/Mozmill
mozmill/docs/         https://developer.mozilla.org/en/Mozmill/
"""

import optparse
import os
import sys
import urllib2

# necessary imports
try:
    import markdown
except ImportError:
    raise ImportError("markdown is not installed, run (e.g.):\neasy_install Markdown")

#DEST='http://developer.mozilla.org/'
DEST='https://developer-stage9.mozilla.org/jhammel'
DIR=os.path.dirname(os.path.abspath(__file__)) # XXX currently unused

def find_readme(directory):
    """find a README file in a directory"""
    # XXX currently unused
    README=['README.md', 'README.txt', 'README']
    for name in README:
        path = os.path.join(directory, name)
        if os.path.exists(path):
            return path

def parse_manifest(filename, directory=None):
    """
    reads a documentation manifest; returns a list of two-tuples:
    [(filename, destination)]
    """
    
    assert os.path.exists(filename) and os.path.isfile(filename), "%s not found" % filename
    
    if directory is None:
        directory = os.path.dirname(os.path.abspath(filename))
    lines = [line.strip() for line in file(filename).readlines()]
    lines = [line for line in lines
             if line and not line.startswith('#')]
    items = []
    for line in lines:
        try:
            f, url = line.split()
            # TODO: include options as third segment (e.g. format=ReST)
        except ValueError:
            raise ValueError("illegal manifest line: '%s'" % line)

        filename = os.path.join(directory, f)
        if os.path.isdir(filename):
            pass # TODO
        else:
            items.append((filename, url))
    return items

def main(args=sys.argv[1:]):

    # parse command line options
    usage = '%prog [options] manifest <manifest> <...>'

    # description formatter
    class PlainDescriptionFormatter(optparse.IndentedHelpFormatter):
        def format_description(self, description):
            if description:
                return description + '\n'
            else:
                return ''
    
    parser = optparse.OptionParser(usage=usage, description=__doc__, formatter=PlainDescriptionFormatter())
    parser.add_option('-d', '--directory', dest='directory',
                      help='render the documentation from this directory')
    parser.add_option('--dest', dest='dest',
                      default=DEST,
                      help='base directory or URL of destination')
    parser.add_option('-u', '--user', dest='user',
                      help='user name')
    parser.add_option('--list', dest='list', action='store_true', default=False,
                      help="list files")
    options, manifests = parser.parse_args(args)

    # print help if no manifests given
    if not args:
        parser.print_help()
        parser.exit()

    # get base url
    if '://' in options.dest:
        baseurl = options.dest
    else:
        baseurl = 'file://' + options.dest

    # read the manifests
    files = []
    for manifest in manifests:
        for item in parse_manifest(manifest):
            if item not in files:
                files.append(item)
    if options.list:
        for item in files:
            print '%s -> %s/%s' % (item[0], baseurl.rstrip('/'), item[1].lstrip('/'))

    # render and upload READMEs
    if options.directory:

        # create a directory if needed
        if os.path.exists(options.directory):
            assert os.path.isdir(options.directory), "'%s' - not a directory" % options.directory
        else:
            os.makedirs(options.directory)

        # TODO render to directory
        
    else:
        # TODO check credentials
        raise NotImplementedError

if __name__ == '__main__':
    main()
